package com.persagy.common.config;

import com.baomidou.mybatisplus.core.incrementer.IdentifierGenerator;
import com.persagy.common.annotation.IdGeneratorPrefix;
import com.persagy.common.utils.IdGenerator;
import com.persagy.common.utils.StringUtil;

/**
 * @version 
 * @description 若需要自定义ID生成规则，请重写对应方法
 * @company persagy
 * @author zhangqiankun
 * @since 2020年9月29日:	上午11:50:05
 */
public class DefaultIdentifierGenerator implements IdentifierGenerator {

	@Override
	public Long nextId(Object entity) {
		return assignLongId();
	}
	
	@Override
	public String nextUUID(Object entity) {
		IdGeneratorPrefix prefix = entity.getClass().getAnnotation(IdGeneratorPrefix.class);
		if (prefix == null || StringUtil.isBlank(prefix.value())) {
			return assignStringId();
		}
        return assignStringId(prefix.value());
    }
	
	public Long assignLongId() {
		return IdGenerator.getSnowId();
	}
	
	public String assignStringId() {
		return IdGenerator.getSnowIdStr();
	}

	public String assignStringId(String prefix) {
		return IdGenerator.getSnowIdStr(prefix);
	}

}
