package com.persagy.common;

import com.persagy.common.redis.aop.LockAop;
import com.persagy.common.redis.base.RedissonTemplate;
import com.persagy.common.redis.boot.RedisBeanContext;
import com.persagy.common.redis.factory.RedissonClientFactory;
import com.persagy.common.redis.properties.CacheManagerProperties;
import com.persagy.common.redis.properties.RedissonProperties;
import com.persagy.common.redis.template.RedissonTemplateImpl;
import org.redisson.api.RedissonClient;
import org.redisson.spring.data.connection.RedissonConnectionFactory;
import org.redisson.spring.starter.RedissonAutoConfiguration;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.serializer.RedisSerializer;

/***
 * @Description: redisson的配置类
 * @author: lijie
 * @Date:2020/9/29 9:41
 * Update By lijie 2020/9/29 9:41
 */
@Configuration
@EnableConfigurationProperties({RedissonProperties.class, CacheManagerProperties.class})
@ConditionalOnClass({RedissonProperties.class})
@EnableCaching
@AutoConfigureBefore({RedissonAutoConfiguration.class})
public class RedisAutoConfigure {
	

    @Autowired
    private RedissonProperties redissonProperties;

    @Bean
    public RedisSerializer<String> redisKeySerializer() {
        return RedisSerializer.string();
    }

    @Bean
    public RedisSerializer<Object> redisValueSerializer() {
        return redissonProperties.getSerializerModel().getSerializerType();
    }
    
    @Bean(name = "redisValueJavaSerializer")
    public RedisSerializer<Object> redisValueJavaSerializer() {
        return RedisSerializer.java();
    }


    @Bean
    @ConditionalOnMissingBean(LockAop.class)
    public LockAop lockAop() {
        return new LockAop();
    }

    @Bean
    @ConditionalOnMissingBean(RedissonTemplate.class)
    public RedissonTemplate redissonTemplate() {
        return new RedissonTemplateImpl(redissonProperties.getDatabase());
    }

    @Bean
    public RedisBeanContext redisBeanContext(ApplicationContext applicationContext) {
        return new RedisBeanContext();
    }

	@Bean(name = "redisTemplate")
    @ConditionalOnMissingBean(name = "redisTemplate")
    public RedisTemplate<String, Object> redisTemplate(RedissonConnectionFactory redissonConnectionFactory,
    		RedisSerializer<String> redisKeySerializer, RedisSerializer<Object> redisValueSerializer) {
        RedisTemplate<String, Object> template = new RedisTemplate<String, Object>();
        template.setConnectionFactory(redissonConnectionFactory);
        
        // 设置value的序列化规则和 key的序列化规则
        template.setKeySerializer(redisKeySerializer);
        template.setValueSerializer(redisValueSerializer);
        template.setHashKeySerializer(redisKeySerializer);
        template.setHashValueSerializer(redisValueSerializer);
        
        template.afterPropertiesSet();
        return template;
    }

    @Bean(name = "stringRedisTemplate")
    @ConditionalOnMissingBean(name = {"stringRedisTemplate"})
    public StringRedisTemplate stringRedisTemplate(RedissonConnectionFactory redissonConnectionFactory) {
        StringRedisTemplate template = new StringRedisTemplate();
        template.setConnectionFactory(redissonConnectionFactory);
        return template;
    }
    
    @Bean
    @ConditionalOnMissingBean(RedisConnectionFactory.class)
    public RedissonConnectionFactory redissonConnectionFactory(RedissonClient redissonClient) {
        return new RedissonConnectionFactory(redissonClient);
    }
    
    @Bean(destroyMethod = "shutdown")
    @ConditionalOnMissingBean(RedissonClient.class)
    public RedissonClient redissonClient() {
    	return RedissonClientFactory.redissonClient(redissonProperties);
    }
    
}
