package com.persagy.common.lock;


import com.persagy.common.model.LockModel;

import java.lang.annotation.*;

/***
 * @Description: 分布式锁的注解
 * @author: lijie
 * @Date:2020/8/18 9:49
 * Update By lijie 2020/8/18 9:49
 */
@Target(ElementType.METHOD)
@Retention(RetentionPolicy.RUNTIME)
@Documented
public @interface Lock {

    /***
     * @Description: 锁的模式:如果不设置,自动模式,当参数只有一个.使用 REENTRANT 参数多个 MULTIPLE
     * @return : com.persagy.enums.LockModel{@link LockModel}
     * @author: lijie
     * @Date:2020/8/18 9:49
     * Update By lijie 2020/8/18 9:49
     */
    LockModel lockModel() default LockModel.AUTO;

    /***
     * @Description: 如果keys有多个,如果不设置,则使用 联锁
     * @return : java.lang.String[]
     * @author: lijie
     * @Date:2020/8/18 9:51
     * Update By lijie 2020/8/18 9:51
     */
    String[] keys() default {};

    /***
     * @Description: key的静态常量:当key的spel的值是LIST,数组时使用+号连接将会被spel认为这个变量是个字符串,只能产生一把锁,
     * 达不到我们的目的,而我们如果又需要一个常量的话.这个参数将会在拼接在每个元素的后面
     * @return : java.lang.String
     * @author: lijie
     * @Date:2020/8/18 9:51
     * Update By lijie 2020/8/18 9:51
     */
    String keyConstant() default "";

    /***
     * @Description: 锁超时时间,默认30000毫秒(可在配置文件全局设置)
     * @return : long
     * @author: lijie
     * @Date:2020/8/18 9:51
     * Update By lijie 2020/8/18 9:51
     */
    long lockWatchdogTimeout() default 0;

    /***
     * @Description: 等待加锁超时时间,默认10000毫秒 -1 则表示一直等待(可在配置文件全局设置)
     * @return : long
     * @author: lijie
     * @Date:2020/8/18 9:52
     * Update By lijie 2020/8/18 9:52
     */
    long attemptTimeout() default 0;
    /***
     * @Description: 加锁库的索引
     * @return : int
     * @author: lijie
     * @Date:2020/8/18 9:52
     * Update By lijie 2020/8/18 9:52
     */
    int databaseIndex() default 0;


}
