package com.persagy.common.redis.base;

import java.util.List;
import java.util.Map;

/***
 * @Description: Hash类型的操作
 * @author: lijie
 * @Date:2020/9/17 9:30
 * Update By lijie 2020/9/17 9:30
 */
public interface HashOperations {

    /***
     * @Description: hash类型存储数据
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @param field : hash#key
     * @param value : 字段值
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    void hset(Integer databaseIndex, String key, String field, String value);

    /***
     * @Description: hash类型存储数据
     * @param key : redis#key
     * @param field : hash#key
     * @param value : 字段值
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    void hset(String key, String field, String value);

    /***
     * @Description: hash类型获取数据
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @param field : hash#key
     * @return : java.lang.String
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    String hget(Integer databaseIndex, String key, String field);

    /***
     * @Description: hash类型获取数据
     * @param key : redis#key
     * @param field : hash#key
     * @return : java.lang.String
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    String hget(String key, String field);

    /***
     * @Description: hash类型删除数据
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @param field : hash#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    Long hdel(Integer databaseIndex, String key, String field);

    /***
     * @Description: hash类型删除数据
     * @param key : redis#key
     * @param field : hash#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    Long hdel(String key, String field);

    /***
     * @Description: hash类型批量添加
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @param map : hashMap对象
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    void hmset(Integer databaseIndex, String key, Map<String, String> map);

    /***
     * @Description: hash类型批量添加
     * @param key : redis#key
     * @param map : hashMap对象
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    void hmset(String key, Map<String, String> map);

    /***
     * @Description: hash类型批量添加
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @param fields : hash#key数组
     * @return : java.util.List<java.lang.String>
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    List<String> hmget(Integer databaseIndex, String key, String... fields);

    /***
     * @Description: hash类型批量添加
     * @param key : redis#key
     * @param fields : hash#key数组
     * @return : java.util.List<java.lang.String>
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    List<String> hmget(String key, String... fields);

    /***
     * @Description: hash类型获取属性个数
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    Long hlen(Integer databaseIndex, String key);

    /***
     * @Description: hash类型获取属性个数
     * @param key : redis#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    Long hlen(String key);

    /***
     * @Description: hash类型获取所有属性
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @return : java.util.Map<java.lang.String,java.lang.String>
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    Map<String, String> hgetAll(Integer databaseIndex, String key);

    /***
     * @Description: hash类型获取所有属性
     * @param key : redis#key
     * @return : java.util.Map<java.lang.String,java.lang.String>
     * @author: lijie
     * @Date:2020/8/18 11:07
     * Update By lijie 2020/8/18 11:07
     */
    Map<String, String> hgetAll(String key);

}
