package com.persagy.common.redis.base;

import java.util.Set;

/***
 * @Description: redis相关的key操作
 * @author: lijie
 * @Date:2020/9/17 9:23
 * Update By lijie 2020/9/17 9:23
 */
public interface KeyOperations {
    /***
     * @Description: 判断redis#key键是否存在
     * @param databaseIndex : 索引库
     * @param key : key
     * @return : boolean
     * @author: lijie
     * @Date:2020/8/18 9:47
     * Update By lijie 2020/8/18 9:47
     */
    Boolean exists(Integer databaseIndex, String key);

    /***
     * @Description: 判断redis#key键是否存在
     * @param key : key
     * @return : boolean
     * @author: lijie
     * @Date:2020/8/18 10:42
     * Update By lijie 2020/8/18 10:42
     */
    Boolean exists(String key);

    /***
     * @Description: 正则获取key
     * @param index : 游标
     * @param regx : 正则表达式
     * @return : java.util.Set<java.lang.String>
     * @author: lijie
     * @Date:2020/8/18 10:51
     * Update By lijie 2020/8/18 10:51
     */
    Set<String> scan(int index, String regx);

    /***
     * @Description: 正则获取key
     * @param regx : 正则表达式
     * @return : java.util.Set<java.lang.String>
     * @author: lijie
     * @Date:2020/8/18 10:51
     * Update By lijie 2020/8/18 10:51
     */
    Set<String> scan(String regx);

    /***
     * @Description: 获取redis#key过期时间
     * @param key : redis#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:03
     * Update By lijie 2020/8/18 11:03
     */
    Long getTtl(Integer databaseIndex, String key);

    /***
     * @Description: 获取redis#key过期时间
     * @param key : redis#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:03
     * Update By lijie 2020/8/18 11:03
     */
    Long getTtl(String key);

    /***
     * @Description: 设置key的过期时间
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @param seconds : 过期时间单位（S）
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 11:14
     * Update By lijie 2020/8/18 11:14
     */
    void setTtl(Integer databaseIndex, String key, int seconds);

    /***
     * @Description: 设置key的过期时间
     * @param key : redis#key
     * @param seconds : 过期时间单位（S）
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 11:14
     * Update By lijie 2020/8/18 11:14
     */
    void setTtl(String key, int seconds);

    /***
     * @Description: 删除redis#key过期时间
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:03
     * Update By lijie 2020/8/18 11:03
     */
    Boolean removeTtl(Integer databaseIndex, String key);

    /***
     * @Description: 删除redis#key过期时间
     * @param key : redis#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:03
     * Update By lijie 2020/8/18 11:03
     */
    Boolean removeTtl(String key);

    /***
     * @Description: 删除redis#key
     * @param key : redis#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:03
     * Update By lijie 2020/8/18 11:03
     */
    Boolean delete(String key);

    /***
     * @Description: 删除redis#key
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:03
     * Update By lijie 2020/8/18 11:03
     */
    Boolean delete(Integer databaseIndex, String key);
}
