package com.persagy.common.redis.base;

import java.util.List;

/***
 * @Description: List类型的操作命令
 * @author: lijie
 * @Date:2020/9/17 9:32
 * Update By lijie 2020/9/17 9:32
 */
public interface ListOperations {

    /***
     * @Description:  从左边入列表
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @param values : 入列表的值
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    void lpush(Integer databaseIndex, String key, String... values);

    /***
     * @Description:  从左边入列表
     * @param key : redis#key
     * @param values : 入列表的值
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    void lpush(String key, String... values);

    /***
     * @Description:  从右边入列表
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @param values : 入列表的值
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    void rpush(Integer databaseIndex, String key, String... values);

    /***
     * @Description:  从右边入列表
     * @param key : redis#key
     * @param values : 入列表的值
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    void rpush(String key, String... values);

    /***
     * @Description:  返回列表长度
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    Long llen(Integer databaseIndex, String key);

    /***
     * @Description:  返回列表长度
     * @param key : redis#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    Long llen(String key);

    /***
     * @Description:  返回index下标的元素
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @param index : 下标
     * @return : java.lang.String
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    String lindex(Integer databaseIndex, String key, long index);

    /***
     * @Description:  返回index下标的元素
     * @param key : redis#key
     * @param index : 下标
     * @return : java.lang.String
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    String lindex(String key, long index);

    /***
     * @Description:  获取key列表所有元素
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @return : java.util.List<java.lang.String>
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    List<String> lrangeAll(Integer databaseIndex, String key);

    /***
     * @Description:  获取key列表所有元素
     * @param key : redis#key
     * @return : java.util.List<java.lang.String>
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    List<String> lrangeAll(String key);

    /***
     * @Description:  获取列表指定范围的元素
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @param start: 开始位置
     * @param end: 结束位置
     * @return : java.util.List<java.lang.String>
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    List<String> lrange(Integer databaseIndex, String key, long start, long end);

    /***
     * @Description:  获取列表指定范围的元素
     * @param key : redis#key
     * @param start: 开始位置
     * @param end: 结束位置
     * @return : java.util.List<java.lang.String>
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    List<String> lrange(String key, long start, long end);

    /***
     * @Description:  弹出列表头部元素
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @return : java.lang.String
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    String lpop(Integer databaseIndex, String key);

    /***
     * @Description:  弹出列表头部元素
     * @param key : redis#key
     * @return : java.lang.String
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    String lpop(String key);

    /***
     * @Description:  弹出列表尾部元素
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @return : java.lang.String
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    String rpop(Integer databaseIndex, String key);

    /***
     * @Description:  弹出列表尾部元素
     * @param key : redis#key
     * @return : java.lang.String
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    String rpop(String key);

}
