package com.persagy.common.redis.base;

import com.persagy.common.lock.DistributedLock;

/**
 * redis 操作函数顶级接口
 */
public interface RedissonTemplate extends KeyOperations, StringOperations, HashOperations, ListOperations, DistributedLock {

    /***
     * @Description:  分布式锁-锁定
     * @param databaseIndex : 库索引
     * @param key: 分布式锁Key
     * @param value: 分布式锁value
     * @param expire: 分布式锁过期时间
     * @return : true-加锁成功/false-加锁失败
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    Boolean tryLock(Integer databaseIndex, String key, String value, long expire);

    /***
     * @Description:  分布式锁-锁定
     * @param key: 分布式锁Key
     * @param value: 分布式锁value
     * @param expire: 分布式锁过期时间
     * @return : true-加锁成功/false-加锁失败
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    Boolean tryLock(String key, String value, long expire);

    /***
     * @Description:  分布式锁-解锁
     * @param databaseIndex: 库索引
     * @param key: 分布式锁Key
     * @param value: 分布式锁value
     * @return : true-解除锁成功/false-解除锁失败
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    Boolean unLock(Integer databaseIndex, String key, String value);

    /***
     * @Description:  分布式锁-解锁
     * @param key: 分布式锁Key
     * @param value: 分布式锁value
     * @return : true-解除锁成功/false-解除锁失败
     * @author: lijie
     * @Date:2020/8/18 11:15
     * Update By lijie 2020/8/18 11:15
     */
    Boolean unLock(String key, String value);
    
    /**
     * redis 分布式锁-锁定
     *
     * @param key          分布式等待锁Key
     * @param expireSecond 持有锁超时毫秒数
     * @param waitSecond   等待锁超时秒数
     * @param flag         线程标识
     * @return true-获取锁成功/false-获取锁失败
     */
    //boolean tryLockWait(String key, long expireSecond, int waitSecond, String flag);

    /**
     * redis 分布式锁-解锁
     *
     * @param key  分布式等待锁Key
     * @param flag 线程标识
     * @return true-解除锁成功/false-解除锁失败
     */
    // boolean unlockWait(String key, String flag);

}
