package com.persagy.common.redis.base;

import java.util.List;

/***
 * @Description: redis的String类型的相关操作
 * @author: lijie
 * @Date:2020/9/17 9:28
 * Update By lijie 2020/9/17 9:28
 */
public interface StringOperations {

    /***
     * @Description: 通过redis#key判断value长度
     * @param databaseIndex : 库索引
     * @param key : key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 10:43
     * Update By lijie 2020/8/18 10:43
     */
    Long length(Integer databaseIndex, String key);

    /***
     * @Description: 通过redis#key判断value长度
     * @param key : key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 10:43
     * Update By lijie 2020/8/18 10:43
     */
    Long length(String key);

    /***
     * @Description: 添加对象
     * @param key : key
     * @param obj : 值对象
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 10:45
     * Update By lijie 2020/8/18 10:45
     */
    void setObj(String key, Object obj);

    /***
     * @Description: 添加对象
     * @param databaseIndex : 库索引
     * @param key : key
     * @param obj : 值对象
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 10:45
     * Update By lijie 2020/8/18 10:45
     */
    void setObj(Integer databaseIndex, String key, Object obj);

    /***
     * @Description: 添加字符串对象
     * @param key : key
     * @param value : 值
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 10:45
     * Update By lijie 2020/8/18 10:45
     */
    void set(String key, String value);
    /***
     * @Description: 添加字符串对象
     * @param databaseIndex : 库索引
     * @param key : key
     * @param value : 值
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 10:45
     * Update By lijie 2020/8/18 10:45
     */
    void set(Integer databaseIndex, String key, String value);

    /***
     * @Description: 添加并设置过期时间
     * @param key : redis#key
     * @param value : 字符串对象
     * @param seconds : 过期时间单位（S）
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 10:46
     * Update By lijie 2020/8/18 10:46
     */
    void setex(String key, String value, int seconds);

    /***
     * @Description: 添加并设置过期时间
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @param value : 字符串对象
     * @param seconds : 过期时间单位（S）
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 10:46
     * Update By lijie 2020/8/18 10:46
     */
    void setex(Integer databaseIndex, String key, String value, int seconds);


    /***
     * @Description: 添加并设置过期时间
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @param obj : 对象
     * @param seconds : 过期时间单位（S）
     * @return : void
     * @author: lijie
     * @Date:2020/8/18 10:46
     * Update By lijie 2020/8/18 10:46
     */
    void setexObj(Integer databaseIndex, String key, Object obj, int seconds);

    /***
     * @Description: 通过key获取值
     * @param key : key
     * @return : java.lang.String
     * @author: lijie
     * @Date:2020/8/18 10:47
     * Update By lijie 2020/8/18 10:47
     */
    String get(String key);

    /***
     * @Description: 通过key获取值
     * @param databaseIndex : 库索引
     * @param key : key
     * @return : java.lang.String
     * @author: lijie
     * @Date:2020/8/18 10:47
     * Update By lijie 2020/8/18 10:47
     */
    String get(Integer databaseIndex, String key);

    /***
     * @Description: 获取多个redis#key的值
     * @param keys : redis#key数组
     * @return : java.util.List<java.lang.String>
     * @author: lijie
     * @Date:2020/8/18 10:49
     * Update By lijie 2020/8/18 10:49
     */
    List<String> get(String... keys);

    /***
     * @Description: 获取多个redis#key的值
     * @param databaseIndex : 库索引
     * @param keys : redis#key数组
     * @return : java.util.List<java.lang.String>
     * @author: lijie
     * @Date:2020/8/18 10:49
     * Update By lijie 2020/8/18 10:49
     */
    List<String> get(Integer databaseIndex, String... keys);

    /***
     * @Description: 获取 redis#key值并转换为目标对象
     * @param key   redis#key
     * @param clazz 目标对象
     * @param <T>   对象类型
     * @return : T
     * @author: lijie
     * @Date:2020/8/18 10:58
     * Update By lijie 2020/8/18 10:58
     */
    <T> T getObj(String key, Class<T> clazz);

    /***
     * @Description: 获取 redis#key值并转换为目标对象
     * @param databaseIndex: 库索引
     * @param key: redis#key
     * @param clazz: 目标对象
     * @param <T>:  对象类型
     * @return : T
     * @author: lijie
     * @Date:2020/8/18 10:58
     * Update By lijie 2020/8/18 10:58
     */
    <T> T getObj(Integer databaseIndex, String key, Class<T> clazz);

    /***
     * @Description: redis#key的值自增1
     * @param databaseIndex : 库索引
     * @param key : redis#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:05
     * Update By lijie 2020/8/18 11:05
     */
    Long incr(Integer databaseIndex, String key);

    /***
     * @Description: redis#key的值自增1
     * @param key : redis#key
     * @return : java.lang.Long
     * @author: lijie
     * @Date:2020/8/18 11:05
     * Update By lijie 2020/8/18 11:05
     */
    Long incr(String key);


}
