package com.persagy.common.redis.boot;

import org.redisson.api.RedissonClient;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.data.redis.core.RedisTemplate;

import java.lang.annotation.Annotation;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 上下文
 *
 */
public class RedisBeanContext implements ApplicationContextAware {

    /**
     * 上下文
     */
    private static ApplicationContext applicationContext;

    /**
     * 默认构造方法，注入上下文,
     * implements ApplicationContextAware 自动调用
     *
     * @param applicationContext
     * @return
     */
    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        RedisBeanContext.applicationContext = applicationContext;
    }

    /**
     * 获取 bean
     *
     * @param clazz
     * @return
     */

    public static <T> T getBean(Class<T> clazz) {
        T obj;
        try {
            //从上下文获取 bean
            obj = applicationContext.getBean(clazz);
        } catch (Exception e) {
            obj = null;
        }
        //返回 bean
        return obj;
    }
    /**
     * 获取 bean
     *
     * @return
     */

    public static <T> T getBeanByName(String clazzName, Class<T> clazz) {
        T obj;
        try {
            //从上下文获取 bean
            obj = (T) applicationContext.getBean(clazzName);
        } catch (Exception e) {
            obj = null;
        }
        //返回 bean
        return obj;
    }


    /**
     * 获取 bean 的类型
     *
     * @param clazz
     * @return
     * @time 2018/5/28 14:03
     */
    public static <T> List<T> getBeansOfType(Class<T> clazz) {
        //声明一个结果
        Map<String, T> map;
        try {
            //获取类型
            map = applicationContext.getBeansOfType(clazz);
        } catch (Exception e) {
            map = null;
        }
        //返回 bean 的类型
        return map == null ? null : new ArrayList<>(map.values());
    }


    /**
     * 获取所有被注解的 bean
     *
     * @return
     */
    public static Map<String, Object> getBeansWithAnnotation(Class<? extends Annotation> annotation) {
        Map<String, Object> map;
        try {
            //获取注解的 bean
            map = applicationContext.getBeansWithAnnotation(annotation);
        } catch (Exception e) {
            map = null;
        }
        return map;
    }

    /***
     * @Description: 获得redissonCliet对象Bean名称
     * @param databaseIndex : 数据库索引
     * @return : java.lang.Integer
     * @author: lijie
     * @Date:2020/8/17 9:58
     * Update By lijie 2020/8/17 9:58
     */
    public static String getRedissonClientBeanName(Integer databaseIndex){
    	return "redissonClient";
        /*switch (databaseIndex){
            case 0:
                return "redissonClient";
            case 1:
                return "redissonClientOne";
            case 2:
                return "redissonClientTwo";
            case 3:
                return "redissonClientThree";
            case 4:
                return "redissonClientFour";
            case 5:
                return "redissonClientFive";
            case 6:
                return "redissonClientSix";
            case 7:
                return "redissonClientSeven";
            case 8:
                return "redissonClientEight";
            case 9:
                return "redissonClientNine";
            case 10:
                return "redissonClientTen";
            case 11:
                return "redissonClientEleven";
            case 12:
                return "redissonClientTwelve";
            case 13:
                return "redissonClientThirteen";
            case 14:
                return "redissonClientFourteen";
            case 15:
                return "redissonClientFifteen";
            default:
                return "redissonClient";
        }*/
    }
    /***
     * @Description: 获取redisTemplate对象
      * @param databaseIndex : 库索引
      * @return : org.springframework.data.redis.core.RedisTemplate
     * @author: lijie
     * @Date:2020/8/18 11:48
     * Update By lijie 2020/8/18 11:48
     */
    /*public static StringRedisTemplate getStringRedisTemplate(Integer databaseIndex){
        return RedisBeanContext.getBeanByName(RedisBeanContext.getStringRedisTemplateBeanName(databaseIndex), StringRedisTemplate.class);
    }*/
    /***
     * @Description: 获取redisTemplate对象
      * @param databaseIndex : 库索引
      * @return : org.springframework.data.redis.core.RedisTemplate
     * @author: lijie
     * @Date:2020/8/18 11:48
     * Update By lijie 2020/8/18 11:48
     */
    public static <T, R> RedisTemplate<T, R> getRedisTemplate(Integer databaseIndex){
        return RedisBeanContext.getBeanByName(RedisBeanContext.getRedisTemplateBeanName(databaseIndex), RedisTemplate.class);
    }
    /***
     * @Description: 获取redisTemplate对象
      * @param databaseIndex : 库索引
      * @return : org.springframework.data.redis.core.RedisTemplate
     * @author: lijie
     * @Date:2020/8/18 11:48
     * Update By lijie 2020/8/18 11:48
     */
    public static RedissonClient getRedissonClient(Integer databaseIndex){
        return RedisBeanContext.getBeanByName(RedisBeanContext.getRedissonClientBeanName(databaseIndex), RedissonClient.class);
    }

    /***
     * @Description: 获得redisTemplate对象
     * @param databaseIndex : 数据库索引
     * @return : java.lang.Integer
     * @author: lijie
     * @Date:2020/8/17 9:58
     * Update By lijie 2020/8/17 9:58
     */
    public static String getRedisTemplateBeanName(Integer databaseIndex){
    	return "redisTemplate";
        /*switch (databaseIndex){
            case 0:
                return "redisTemplate";
            case 1:
                return "redisTemplateOne";
            case 2:
                return "redisTemplateTwo";
            case 3:
                return "redisTemplateThree";
            case 4:
                return "redisTemplateFour";
            case 5:
                return "redisTemplateFive";
            case 6:
                return "redisTemplateSix";
            case 7:
                return "redisTemplateSeven";
            case 8:
                return "redisTemplateEight";
            case 9:
                return "redisTemplateNine";
            case 10:
                return "redisTemplateTen";
            case 11:
                return "redisTemplateEleven";
            case 12:
                return "redisTemplateTwelve";
            case 13:
                return "redisTemplateThirteen";
            case 14:
                return "redisTemplateFourteen";
            case 15:
                return "redisTemplateFifteen";
            default:
                return "redisTemplate";
        }*/
    }
}
