package com.persagy.common.redis.factory;

import com.persagy.common.redis.properties.MultipleServerConfig;
import com.persagy.common.redis.properties.RedissonProperties;
import org.redisson.Redisson;
import org.redisson.api.RedissonClient;
import org.redisson.client.codec.Codec;
import org.redisson.config.ClusterServersConfig;
import org.redisson.config.Config;
import org.redisson.connection.balancer.LoadBalancer;

/**
 * @version
 * @description
 * @company persagy
 * @author zhangqiankun
 * @since 2020年10月14日: 下午3:09:49
 */
public class ClusterRedissionConfig {

	public static RedissonClient redissonClient(RedissonProperties redissonProperties) {
		return Redisson.create(initConfig(redissonProperties));
	}

	private static Config initConfig(RedissonProperties redissonProperties) {
		Config config = new Config();
		try {
			config.setCodec((Codec) Class.forName(redissonProperties.getCodec().getCodecType()).newInstance());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		config.setTransportMode(redissonProperties.getTransportMode());
		if (redissonProperties.getThreads() != null) {
			config.setThreads(redissonProperties.getThreads());
		}
		if (redissonProperties.getNettyThreads() != null) {
			config.setNettyThreads(redissonProperties.getNettyThreads());
		}
		config.setReferenceEnabled(redissonProperties.getReferenceEnabled());
		config.setLockWatchdogTimeout(redissonProperties.getLockWatchdogTimeout());
		config.setKeepPubSubOrder(redissonProperties.getKeepPubSubOrder());
		config.setUseScriptCache(redissonProperties.getUseScriptCache());
		config.setMinCleanUpDelay(redissonProperties.getMinCleanUpDelay());
		config.setMaxCleanUpDelay(redissonProperties.getMaxCleanUpDelay());

		MultipleServerConfig multipleServerConfig = redissonProperties.getMultipleServerConfig();

		ClusterServersConfig clusterServersConfig = config.useClusterServers();
		clusterServersConfig.setScanInterval(multipleServerConfig.getScanInterval());
		clusterServersConfig
				.setSlaveConnectionMinimumIdleSize(multipleServerConfig.getSlaveConnectionMinimumIdleSize());
		clusterServersConfig.setSlaveConnectionPoolSize(multipleServerConfig.getSlaveConnectionPoolSize());
		clusterServersConfig
				.setFailedSlaveReconnectionInterval(multipleServerConfig.getFailedSlaveReconnectionInterval());
		clusterServersConfig.setFailedSlaveCheckInterval(multipleServerConfig.getFailedSlaveCheckInterval());
		clusterServersConfig
				.setMasterConnectionMinimumIdleSize(multipleServerConfig.getMasterConnectionMinimumIdleSize());
		clusterServersConfig.setMasterConnectionPoolSize(multipleServerConfig.getMasterConnectionPoolSize());
		clusterServersConfig.setReadMode(multipleServerConfig.getReadMode());
		clusterServersConfig.setSubscriptionMode(multipleServerConfig.getSubscriptionMode());
		clusterServersConfig.setSubscriptionConnectionMinimumIdleSize(
				multipleServerConfig.getSubscriptionConnectionMinimumIdleSize());
		clusterServersConfig
				.setSubscriptionConnectionPoolSize(multipleServerConfig.getSubscriptionConnectionPoolSize());
		clusterServersConfig.setDnsMonitoringInterval(multipleServerConfig.getDnsMonitoringInterval());
		try {
			clusterServersConfig.setLoadBalancer(
					(LoadBalancer) Class.forName(multipleServerConfig.getLoadBalancer()).newInstance());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		for (String nodeAddress : multipleServerConfig.getNodeAddresses()) {
			clusterServersConfig.addNodeAddress(RedissonClientFactory.prefixAddress(nodeAddress));
		}
		// clusterServersConfig.setPingTimeout(redissonProperties.getPingTimeout());
		clusterServersConfig.setClientName(redissonProperties.getClientName());
		clusterServersConfig.setConnectTimeout(redissonProperties.getConnectTimeout());
		clusterServersConfig.setIdleConnectionTimeout(redissonProperties.getIdleConnectionTimeout());
		clusterServersConfig.setKeepAlive(redissonProperties.getKeepAlive());
		clusterServersConfig.setPassword(redissonProperties.getPassword());
		clusterServersConfig.setPingConnectionInterval(redissonProperties.getPingConnectionInterval());
		clusterServersConfig.setRetryAttempts(redissonProperties.getRetryAttempts());
		clusterServersConfig.setRetryInterval(redissonProperties.getRetryInterval());
		clusterServersConfig
				.setSslEnableEndpointIdentification(redissonProperties.getSslEnableEndpointIdentification());
		try {
			if (null != redissonProperties.getSslKeystore()) {
				clusterServersConfig.setSslKeystore(redissonProperties.getSslKeystore().toURL());
			}
			if (null != redissonProperties.getSslTruststore()) {
				clusterServersConfig.setSslTruststore(redissonProperties.getSslTruststore().toURL());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		clusterServersConfig.setSslKeystorePassword(redissonProperties.getSslKeystorePassword());
		clusterServersConfig.setSslProvider(redissonProperties.getSslProvider());
		clusterServersConfig.setSslTruststorePassword(redissonProperties.getSslTruststorePassword());
		clusterServersConfig.setSubscriptionsPerConnection(redissonProperties.getSubscriptionsPerConnection());
		clusterServersConfig.setTcpNoDelay(redissonProperties.getTcpNoDelay());
		clusterServersConfig.setTimeout(redissonProperties.getTimeout());

		return config;
	}

}
