package com.persagy.common.redis.factory;

import com.persagy.common.redis.properties.MultipleServerConfig;
import com.persagy.common.redis.properties.RedissonProperties;
import org.redisson.Redisson;
import org.redisson.api.RedissonClient;
import org.redisson.client.codec.Codec;
import org.redisson.config.Config;
import org.redisson.config.MasterSlaveServersConfig;
import org.redisson.connection.balancer.LoadBalancer;

/**
 * @version
 * @description
 * @company persagy
 * @author zhangqiankun
 * @since 2020年10月14日: 下午3:09:49
 */
public class MasterSlaveRedissionConfig {

	public static RedissonClient redissonClient(RedissonProperties redissonProperties) {
		return Redisson.create(initConfig(redissonProperties));
	}

	private static Config initConfig(RedissonProperties redissonProperties) {
		Config config = new Config();
		try {
			config.setCodec((Codec) Class.forName(redissonProperties.getCodec().getCodecType()).newInstance());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		config.setTransportMode(redissonProperties.getTransportMode());
		if (redissonProperties.getThreads() != null) {
			config.setThreads(redissonProperties.getThreads());
		}
		if (redissonProperties.getNettyThreads() != null) {
			config.setNettyThreads(redissonProperties.getNettyThreads());
		}
		config.setReferenceEnabled(redissonProperties.getReferenceEnabled());
		config.setLockWatchdogTimeout(redissonProperties.getLockWatchdogTimeout());
		config.setKeepPubSubOrder(redissonProperties.getKeepPubSubOrder());
		config.setUseScriptCache(redissonProperties.getUseScriptCache());
		config.setMinCleanUpDelay(redissonProperties.getMinCleanUpDelay());
		config.setMaxCleanUpDelay(redissonProperties.getMaxCleanUpDelay());

		MultipleServerConfig multipleServerConfig = redissonProperties.getMultipleServerConfig();

		MasterSlaveServersConfig masterSlaveServersConfig = config.useMasterSlaveServers();
		masterSlaveServersConfig.setDatabase(redissonProperties.getDatabase());
		masterSlaveServersConfig
				.setSlaveConnectionMinimumIdleSize(multipleServerConfig.getSlaveConnectionMinimumIdleSize());
		masterSlaveServersConfig.setSlaveConnectionPoolSize(multipleServerConfig.getSlaveConnectionPoolSize());
		masterSlaveServersConfig
				.setFailedSlaveReconnectionInterval(multipleServerConfig.getFailedSlaveReconnectionInterval());
		masterSlaveServersConfig.setFailedSlaveCheckInterval(multipleServerConfig.getFailedSlaveCheckInterval());
		masterSlaveServersConfig
				.setMasterConnectionMinimumIdleSize(multipleServerConfig.getMasterConnectionMinimumIdleSize());
		masterSlaveServersConfig.setMasterConnectionPoolSize(multipleServerConfig.getMasterConnectionPoolSize());
		masterSlaveServersConfig.setReadMode(multipleServerConfig.getReadMode());
		masterSlaveServersConfig.setSubscriptionMode(multipleServerConfig.getSubscriptionMode());
		masterSlaveServersConfig.setSubscriptionConnectionMinimumIdleSize(
				multipleServerConfig.getSubscriptionConnectionMinimumIdleSize());
		masterSlaveServersConfig
				.setSubscriptionConnectionPoolSize(multipleServerConfig.getSubscriptionConnectionPoolSize());
		masterSlaveServersConfig.setDnsMonitoringInterval(multipleServerConfig.getDnsMonitoringInterval());
		try {
			masterSlaveServersConfig.setLoadBalancer(
					(LoadBalancer) Class.forName(multipleServerConfig.getLoadBalancer()).newInstance());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		int index = 0;
		for (String nodeAddress : multipleServerConfig.getNodeAddresses()) {
			if (index++ == 0) {
				masterSlaveServersConfig.setMasterAddress(RedissonClientFactory.prefixAddress(nodeAddress));
			} else {
				masterSlaveServersConfig.addSlaveAddress(RedissonClientFactory.prefixAddress(nodeAddress));
			}
		}
		masterSlaveServersConfig.setClientName(redissonProperties.getClientName());
		masterSlaveServersConfig.setConnectTimeout(redissonProperties.getConnectTimeout());
		masterSlaveServersConfig.setIdleConnectionTimeout(redissonProperties.getIdleConnectionTimeout());
		masterSlaveServersConfig.setKeepAlive(redissonProperties.getKeepAlive());
		masterSlaveServersConfig.setPassword(redissonProperties.getPassword());
		masterSlaveServersConfig.setPingConnectionInterval(redissonProperties.getPingConnectionInterval());
		masterSlaveServersConfig.setRetryAttempts(redissonProperties.getRetryAttempts());
		masterSlaveServersConfig.setRetryInterval(redissonProperties.getRetryInterval());
		masterSlaveServersConfig
				.setSslEnableEndpointIdentification(redissonProperties.getSslEnableEndpointIdentification());
		try {
			if (null != redissonProperties.getSslKeystore()) {
				masterSlaveServersConfig.setSslKeystore(redissonProperties.getSslKeystore().toURL());
			}
			if (null != redissonProperties.getSslTruststore()) {
				masterSlaveServersConfig.setSslTruststore(redissonProperties.getSslTruststore().toURL());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		masterSlaveServersConfig.setSslKeystorePassword(redissonProperties.getSslKeystorePassword());
		masterSlaveServersConfig.setSslProvider(redissonProperties.getSslProvider());
		masterSlaveServersConfig.setSslTruststorePassword(redissonProperties.getSslTruststorePassword());
		masterSlaveServersConfig.setSubscriptionsPerConnection(redissonProperties.getSubscriptionsPerConnection());
		masterSlaveServersConfig.setTcpNoDelay(redissonProperties.getTcpNoDelay());
		masterSlaveServersConfig.setTimeout(redissonProperties.getTimeout());
		
		return config;
	}

}
