package com.persagy.common.redis.factory;

import com.persagy.common.redis.properties.MultipleServerConfig;
import com.persagy.common.redis.properties.RedissonProperties;
import org.redisson.Redisson;
import org.redisson.api.RedissonClient;
import org.redisson.client.codec.Codec;
import org.redisson.config.Config;
import org.redisson.config.ReplicatedServersConfig;
import org.redisson.connection.balancer.LoadBalancer;

/**
 * @version
 * @description
 * @company persagy
 * @author zhangqiankun
 * @since 2020年10月14日: 下午3:09:49
 */
public class ReplicatedRedissionConfig {

	public static RedissonClient redissonClient(RedissonProperties redissonProperties) {
		return Redisson.create(initConfig(redissonProperties));
	}

	private static Config initConfig(RedissonProperties redissonProperties) {
		Config config = new Config();
		try {
			config.setCodec((Codec) Class.forName(redissonProperties.getCodec().getCodecType()).newInstance());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		config.setTransportMode(redissonProperties.getTransportMode());
		if (redissonProperties.getThreads() != null) {
			config.setThreads(redissonProperties.getThreads());
		}
		if (redissonProperties.getNettyThreads() != null) {
			config.setNettyThreads(redissonProperties.getNettyThreads());
		}
		config.setReferenceEnabled(redissonProperties.getReferenceEnabled());
		config.setLockWatchdogTimeout(redissonProperties.getLockWatchdogTimeout());
		config.setKeepPubSubOrder(redissonProperties.getKeepPubSubOrder());
		config.setUseScriptCache(redissonProperties.getUseScriptCache());
		config.setMinCleanUpDelay(redissonProperties.getMinCleanUpDelay());
		config.setMaxCleanUpDelay(redissonProperties.getMaxCleanUpDelay());

		MultipleServerConfig multipleServerConfig = redissonProperties.getMultipleServerConfig();
		
		ReplicatedServersConfig replicatedServersConfig = config.useReplicatedServers();
		replicatedServersConfig.setDatabase(redissonProperties.getDatabase());
		replicatedServersConfig.setScanInterval(multipleServerConfig.getScanInterval());
		replicatedServersConfig
				.setSlaveConnectionMinimumIdleSize(multipleServerConfig.getSlaveConnectionMinimumIdleSize());
		replicatedServersConfig.setSlaveConnectionPoolSize(multipleServerConfig.getSlaveConnectionPoolSize());
		replicatedServersConfig
				.setFailedSlaveReconnectionInterval(multipleServerConfig.getFailedSlaveReconnectionInterval());
		replicatedServersConfig.setFailedSlaveCheckInterval(multipleServerConfig.getFailedSlaveCheckInterval());
		replicatedServersConfig
				.setMasterConnectionMinimumIdleSize(multipleServerConfig.getMasterConnectionMinimumIdleSize());
		replicatedServersConfig.setMasterConnectionPoolSize(multipleServerConfig.getMasterConnectionPoolSize());
		replicatedServersConfig.setReadMode(multipleServerConfig.getReadMode());
		replicatedServersConfig.setSubscriptionMode(multipleServerConfig.getSubscriptionMode());
		replicatedServersConfig.setSubscriptionConnectionMinimumIdleSize(
				multipleServerConfig.getSubscriptionConnectionMinimumIdleSize());
		replicatedServersConfig
				.setSubscriptionConnectionPoolSize(multipleServerConfig.getSubscriptionConnectionPoolSize());
		replicatedServersConfig.setDnsMonitoringInterval(multipleServerConfig.getDnsMonitoringInterval());
		try {
			replicatedServersConfig.setLoadBalancer(
					(LoadBalancer) Class.forName(multipleServerConfig.getLoadBalancer()).newInstance());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		for (String nodeAddress : multipleServerConfig.getNodeAddresses()) {
			replicatedServersConfig.addNodeAddress(RedissonClientFactory.prefixAddress(nodeAddress));
		}
		// replicatedServersConfig.setPingTimeout(redissonProperties.getPingTimeout());
		replicatedServersConfig.setClientName(redissonProperties.getClientName());
		replicatedServersConfig.setConnectTimeout(redissonProperties.getConnectTimeout());
		replicatedServersConfig.setIdleConnectionTimeout(redissonProperties.getIdleConnectionTimeout());
		replicatedServersConfig.setKeepAlive(redissonProperties.getKeepAlive());
		replicatedServersConfig.setPassword(redissonProperties.getPassword());
		replicatedServersConfig.setPingConnectionInterval(redissonProperties.getPingConnectionInterval());
		replicatedServersConfig.setRetryAttempts(redissonProperties.getRetryAttempts());
		replicatedServersConfig.setRetryInterval(redissonProperties.getRetryInterval());
		replicatedServersConfig
				.setSslEnableEndpointIdentification(redissonProperties.getSslEnableEndpointIdentification());
		try {
			if (null != redissonProperties.getSslKeystore()) {
				replicatedServersConfig.setSslKeystore(redissonProperties.getSslKeystore().toURL());
			}
			if (null != redissonProperties.getSslTruststore()) {
				replicatedServersConfig.setSslTruststore(redissonProperties.getSslTruststore().toURL());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		replicatedServersConfig.setSslKeystorePassword(redissonProperties.getSslKeystorePassword());
		replicatedServersConfig.setSslProvider(redissonProperties.getSslProvider());
		replicatedServersConfig.setSslTruststorePassword(redissonProperties.getSslTruststorePassword());
		replicatedServersConfig.setSubscriptionsPerConnection(redissonProperties.getSubscriptionsPerConnection());
		replicatedServersConfig.setTcpNoDelay(redissonProperties.getTcpNoDelay());
		replicatedServersConfig.setTimeout(redissonProperties.getTimeout());
		return config;
	}

}
