package com.persagy.common.redis.factory;

import com.persagy.common.redis.properties.MultipleServerConfig;
import com.persagy.common.redis.properties.RedissonProperties;
import org.redisson.Redisson;
import org.redisson.api.RedissonClient;
import org.redisson.client.codec.Codec;
import org.redisson.config.Config;
import org.redisson.config.SentinelServersConfig;
import org.redisson.connection.balancer.LoadBalancer;

/**
 * @version
 * @description
 * @company persagy
 * @author zhangqiankun
 * @since 2020年10月14日: 下午3:09:49
 */
public class SentinelRedissionConfig {

	public static RedissonClient redissonClient(RedissonProperties redissonProperties) {
		return Redisson.create(initConfig(redissonProperties));
	}

	private static Config initConfig(RedissonProperties redissonProperties) {
		Config config = new Config();
		try {
			config.setCodec((Codec) Class.forName(redissonProperties.getCodec().getCodecType()).newInstance());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		config.setTransportMode(redissonProperties.getTransportMode());
		if (redissonProperties.getThreads() != null) {
			config.setThreads(redissonProperties.getThreads());
		}
		if (redissonProperties.getNettyThreads() != null) {
			config.setNettyThreads(redissonProperties.getNettyThreads());
		}
		config.setReferenceEnabled(redissonProperties.getReferenceEnabled());
		config.setLockWatchdogTimeout(redissonProperties.getLockWatchdogTimeout());
		config.setKeepPubSubOrder(redissonProperties.getKeepPubSubOrder());
		config.setUseScriptCache(redissonProperties.getUseScriptCache());
		config.setMinCleanUpDelay(redissonProperties.getMinCleanUpDelay());
		config.setMaxCleanUpDelay(redissonProperties.getMaxCleanUpDelay());

		MultipleServerConfig multipleServerConfig = redissonProperties.getMultipleServerConfig();
		
		SentinelServersConfig sentinelServersConfig = config.useSentinelServers();
		sentinelServersConfig.setDatabase(redissonProperties.getDatabase());
		sentinelServersConfig.setMasterName(multipleServerConfig.getMasterName());
		sentinelServersConfig.setScanInterval(multipleServerConfig.getScanInterval());
		sentinelServersConfig
				.setSlaveConnectionMinimumIdleSize(multipleServerConfig.getSlaveConnectionMinimumIdleSize());
		sentinelServersConfig.setSlaveConnectionPoolSize(multipleServerConfig.getSlaveConnectionPoolSize());
		sentinelServersConfig
				.setFailedSlaveReconnectionInterval(multipleServerConfig.getFailedSlaveReconnectionInterval());
		sentinelServersConfig.setFailedSlaveCheckInterval(multipleServerConfig.getFailedSlaveCheckInterval());
		sentinelServersConfig
				.setMasterConnectionMinimumIdleSize(multipleServerConfig.getMasterConnectionMinimumIdleSize());
		sentinelServersConfig.setMasterConnectionPoolSize(multipleServerConfig.getMasterConnectionPoolSize());
		sentinelServersConfig.setReadMode(multipleServerConfig.getReadMode());
		sentinelServersConfig.setSubscriptionMode(multipleServerConfig.getSubscriptionMode());
		sentinelServersConfig.setSubscriptionConnectionMinimumIdleSize(
				multipleServerConfig.getSubscriptionConnectionMinimumIdleSize());
		sentinelServersConfig
				.setSubscriptionConnectionPoolSize(multipleServerConfig.getSubscriptionConnectionPoolSize());
		sentinelServersConfig.setDnsMonitoringInterval(multipleServerConfig.getDnsMonitoringInterval());
		try {
			sentinelServersConfig.setLoadBalancer(
					(LoadBalancer) Class.forName(multipleServerConfig.getLoadBalancer()).newInstance());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		for (String nodeAddress : multipleServerConfig.getNodeAddresses()) {
			sentinelServersConfig.addSentinelAddress(RedissonClientFactory.prefixAddress(nodeAddress));
		}
		// sentinelServersConfig.setPingTimeout(redissonProperties.getPingTimeout());
		sentinelServersConfig.setClientName(redissonProperties.getClientName());
		sentinelServersConfig.setConnectTimeout(redissonProperties.getConnectTimeout());
		sentinelServersConfig.setIdleConnectionTimeout(redissonProperties.getIdleConnectionTimeout());
		sentinelServersConfig.setKeepAlive(redissonProperties.getKeepAlive());
		sentinelServersConfig.setPassword(redissonProperties.getPassword());
		sentinelServersConfig.setPingConnectionInterval(redissonProperties.getPingConnectionInterval());
		sentinelServersConfig.setRetryAttempts(redissonProperties.getRetryAttempts());
		sentinelServersConfig.setRetryInterval(redissonProperties.getRetryInterval());
		sentinelServersConfig
				.setSslEnableEndpointIdentification(redissonProperties.getSslEnableEndpointIdentification());
		try {
			if (null != redissonProperties.getSslKeystore()) {
				sentinelServersConfig.setSslKeystore(redissonProperties.getSslKeystore().toURL());
			}
			if (null != redissonProperties.getSslTruststore()) {
				sentinelServersConfig.setSslTruststore(redissonProperties.getSslTruststore().toURL());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		sentinelServersConfig.setSslKeystorePassword(redissonProperties.getSslKeystorePassword());
		sentinelServersConfig.setSslProvider(redissonProperties.getSslProvider());
		sentinelServersConfig.setSslTruststorePassword(redissonProperties.getSslTruststorePassword());
		sentinelServersConfig.setSubscriptionsPerConnection(redissonProperties.getSubscriptionsPerConnection());
		sentinelServersConfig.setTcpNoDelay(redissonProperties.getTcpNoDelay());
		sentinelServersConfig.setTimeout(redissonProperties.getTimeout());

		return config;
	}

}
