package com.persagy.common.redis.factory;

import com.persagy.common.redis.properties.RedissonProperties;
import org.redisson.Redisson;
import org.redisson.api.RedissonClient;
import org.redisson.client.codec.Codec;
import org.redisson.config.Config;
import org.redisson.config.SingleServerConfig;

/**
 * @version 
 * @description 
 * @company persagy
 * @author zhangqiankun
 * @since 2020年10月14日:	下午3:09:49
 */
public class SingleRedissionConfig {
	
	/**
	 * 
	 * @param redissonProperties
	 * @return
	 */
	public static RedissonClient redissonClient(RedissonProperties redissonProperties){
        return Redisson.create(initConfig(redissonProperties));
    }
	
	private static Config initConfig(RedissonProperties redissonProperties) {
		Config config = new Config();
		try {
			config.setCodec((Codec) Class.forName(redissonProperties.getCodec().getCodecType()).newInstance());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		config.setTransportMode(redissonProperties.getTransportMode());
		if (redissonProperties.getThreads() != null) {
			config.setThreads(redissonProperties.getThreads());
		}
		if (redissonProperties.getNettyThreads() != null) {
			config.setNettyThreads(redissonProperties.getNettyThreads());
		}
		config.setReferenceEnabled(redissonProperties.getReferenceEnabled());
		config.setLockWatchdogTimeout(redissonProperties.getLockWatchdogTimeout());
		config.setKeepPubSubOrder(redissonProperties.getKeepPubSubOrder());
		config.setUseScriptCache(redissonProperties.getUseScriptCache());
		config.setMinCleanUpDelay(redissonProperties.getMinCleanUpDelay());
		config.setMaxCleanUpDelay(redissonProperties.getMaxCleanUpDelay());
		
		SingleServerConfig singleServerConfig = config.useSingleServer();
        com.persagy.common.redis.properties.SingleServerConfig param = redissonProperties.getSingleServerConfig();
        singleServerConfig.setAddress(RedissonClientFactory.prefixAddress(param.getAddress()));
        singleServerConfig.setConnectionMinimumIdleSize(param.getConnectionMinimumIdleSize());
        singleServerConfig.setConnectionPoolSize(param.getConnectionPoolSize());
        singleServerConfig.setDatabase(redissonProperties.getDatabase());
        singleServerConfig.setDnsMonitoringInterval(param.getDnsMonitoringInterval());
        singleServerConfig.setSubscriptionConnectionMinimumIdleSize(param.getSubscriptionConnectionMinimumIdleSize());
        singleServerConfig.setSubscriptionConnectionPoolSize(param.getSubscriptionConnectionPoolSize());
        // singleServerConfig.setPingTimeout(redissonProperties.getPingTimeout());
        singleServerConfig.setClientName(redissonProperties.getClientName());
        singleServerConfig.setConnectTimeout(redissonProperties.getConnectTimeout());
        singleServerConfig.setIdleConnectionTimeout(redissonProperties.getIdleConnectionTimeout());
        singleServerConfig.setKeepAlive(redissonProperties.getKeepAlive());
        singleServerConfig.setPassword(redissonProperties.getPassword());
        singleServerConfig.setPingConnectionInterval(redissonProperties.getPingConnectionInterval());
        singleServerConfig.setRetryAttempts(redissonProperties.getRetryAttempts());
        singleServerConfig.setRetryInterval(redissonProperties.getRetryInterval());
        singleServerConfig.setSslEnableEndpointIdentification(redissonProperties.getSslEnableEndpointIdentification());
        try{
            if (null!=redissonProperties.getSslKeystore()){
                singleServerConfig.setSslKeystore(redissonProperties.getSslKeystore().toURL());
            }
            if (null!=redissonProperties.getSslTruststore()){
                singleServerConfig.setSslTruststore(redissonProperties.getSslTruststore().toURL());
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        singleServerConfig.setSslKeystorePassword(redissonProperties.getSslKeystorePassword());
        singleServerConfig.setSslProvider(redissonProperties.getSslProvider());
        singleServerConfig.setSslTruststorePassword(redissonProperties.getSslTruststorePassword());
        singleServerConfig.setSubscriptionsPerConnection(redissonProperties.getSubscriptionsPerConnection());
        singleServerConfig.setTcpNoDelay(redissonProperties.getTcpNoDelay());
        singleServerConfig.setTimeout(redissonProperties.getTimeout());
        
        return config;
	}
	
}
