package com.persagy.common.redis.model;

import com.alibaba.fastjson.support.spring.FastJsonRedisSerializer;
import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.JdkSerializationRedisSerializer;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.data.redis.serializer.StringRedisSerializer;

public enum  SerializerModel {
    /**StringRedisSerializer,字符串序列化器.key必须是字符串*/
    GENERIC_TO_STRING_SERIALIZER(new StringRedisSerializer()),
    
    /**FastJsonRedisSerializer,fastJson的序列化器*/
    FAST_JSON_REDIS_SERIALIZER(new FastJsonRedisSerializer<Object>(Object.class)),
    
    /**Jackson2JsonRedisSerializer,Jackson2Json的序列化器*/
    JACKSON_TO_JSON_REDIS_SERIALIZER(new Jackson2JsonRedisSerializer<Object>(Object.class)),
    
    /**redis默认的jdk序列化器,严重浪费redis内存的一款*/
    CBOR_JACKSON_CODEC(new JdkSerializationRedisSerializer());

    private RedisSerializer<Object> serializerType;
    
    SerializerModel(RedisSerializer serializerType) {
    	this.serializerType = serializerType;
    }

    public RedisSerializer<Object> getSerializerType() {
    	if (serializerType instanceof Jackson2JsonRedisSerializer) {
    		ObjectMapper objectMapper = new ObjectMapper();
            objectMapper.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.ANY);
            objectMapper.enableDefaultTyping(ObjectMapper.DefaultTyping.NON_FINAL);
    		((Jackson2JsonRedisSerializer<Object>) serializerType).setObjectMapper(objectMapper);
		}
        return serializerType;
    }
}
